/* Jatha -- bringing to Java some of the power of lisp.
 * Copyright (C) 2001  Kimberley Burchett   kimbly@pobox.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jatha;

import java.io.*;

/**
 * This is the base class for all macros.
 */
public class Macro {
	/**
	 * Contains helper functions for macros.
	 */
	protected static MacroUtil util = new MacroUtil();

	/**
	 * Most macros use the other version of expand.  When a file contains a 
	 * call to a macro, the Expander passes the macro arguments to the
	 * appropriate Macro object, via this function.
	 * 
	 * <p>This version of expand needs to do its own argument processing.
	 * For example, if your macro takes two arguments separated by a comma like 
	 * <code>@foo(a, b)</code>, this version of expand will be passed "a, b".  
         * The other version of expand would get {"a", "b"}.  If your Macro is like 
	 * most macros, you should override the other version and pretend this one 
	 * doesn't exist.
	 * 
	 * <p>The default implementation of this method calls
	 * <code>MacroUtil.splitString(arg, ',', true, true, true, true)</code>
	 * and passes the result to the other version of expand.  In English, 
	 * that means that it breaks the raw argument string into a bunch of 
	 * separate arguments, separated by commas.  Each argument has any leading
	 * or trailing whitespace removed, and commas that are inside parenthesis
	 * or double-quotes are not split into separate arguments.
	 * 
	 * @param arg The raw, unsplit argument to the macro.
	 */
	public void expand(String arg, Writer out, Expander expander)
		throws IOException
	{
		expand(util.splitString(arg, ',', true, true, true, true), out, expander);
	}

	/**
	 * This is the more convenient version of expand.  The arguments have
	 * already been split and trimmed.  For example, if your macro takes two
	 * arguments separated by a comma like @foo(a, b), this version of expand
	 * will get {"a", "b"} instead of "a, b".  Nested parentheses and strings 
	 * are preserved whole; for example, <code>@foo(a(b,c), x"y,z")</code> 
	 * becomes {"a(b,c)", "x\"y,z\""}.  Note that in that example, there are 
	 * only two arguments, not four.
	 *
	 * <p>If you want to parse the arguments yourself, you should use the 
	 * other version of expand.
	 * 
	 * <p>The default implementation throws an error, complaining that expand()
	 * hasn't been implemented.
	 */
	public void expand(String[] args, Writer out, Expander expander)
		throws IOException
	{
		expander.error(getClass().getName()+".expand() not implemented.");
	}
}
