/* Jatha -- bringing to Java some of the power of lisp.
 * Copyright (C) 2001  Kimberley Burchett   kimbly@pobox.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jatha;

import java.io.*;

/**
 * Usage: java jatha.Main [ -all | -quiet ] [-dir &lt;directory&gt;]
 *
 * <p>This expands the macros in all the *.jatha files in the given 
 * directory, and stores the results in corresponding *.java files.  
 * If no directory is given via the -dir switch, the current directory
 * is used.  File extensions are case insensitive, so both FOO.JMAC and 
 * foo.jatha will work.
 * 
 * <p>By default, if the java file is more recent than the jatha file, we 
 * skip that file (this really speeds things up if you only changed one 
 * file).  If the -all option is specified, all files are recreated, even 
 * if the generated file is more recent than the jatha file (this is useful 
 * if you change the implementation of one of the macros used in the files).
 *
 * <p>If -all is not specified, and a file is up to date, a message
 * will be printed to the screen saying that the file wasn't regenerated.
 * To turn off this message, use -quiet.  -quiet and -all are mutually
 * exclusive.
 */
public class Main {
	public static void main(String[] args) {
		String usage = "Jatha version 1.0\n" +
		               "Usage: java jatha.Main [-all | -quiet] [-dir <directory>]";

		// if true, forces all files to be regenerated.  otherwise, they're 
		// only regenerated if the input file is newer than the output file.
		boolean all = false;

		// if true, silences messages about up-to-date files not being
		// regenerated.
		boolean quiet = false;

		// the directory in which to search for files
		String directory = ".";

		for (int i=0; i < args.length; ++i) {
			if (args[i].equalsIgnoreCase("-all"))
				all = true;
			else if (args[i].equalsIgnoreCase("-quiet"))
				quiet = true;
			else if (args[i].equalsIgnoreCase("-dir") ||
			         args[i].equalsIgnoreCase("-directory"))
			 {
				if (i+1 >= args.length) {
					System.out.println(usage);
					return;
				}
				else {
					directory = args[i+1];
					++i;
				}
			}
			else {
				System.out.println(usage);
				return;
			}
		}

		Expander expander = new Expander();
		expander.registerMacroFactory(new DefaultMacroFactory(), false);

		File curDir = new File(directory);
		String[] filenames = curDir.list();
		for (int i=0; i < filenames.length; ++i) {
			String filename = filenames[i];
			if (filename.length() < 5)
				continue;
			// can't use endsWith() because it's case sensitive
			String extension = filename.substring(filename.length() - 6);
			if (extension.equalsIgnoreCase(".jatha")) {
				String baseFilename = MacroUtil.clip(filename, 0, 6);
				expander.processFile(new File(directory, baseFilename+".jatha"), 
					new File(directory, baseFilename), all, quiet);
			}
		}
		System.out.flush();
	}
}
